﻿using System;
using HeuristicLab.Encodings.RealVectorEncoding;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  /// <summary>
  /// The inverted generational Distance is defined as the mean of a all d[i]^(1/p),
  ///  where d[i] is the minimal distance the ith point of the optimal pareto front has to any point in the evaluated front.   
  /// </summary>
  public class InvertedGenerationalDistance : IMultiObjectiveDistance {
    private double p;

    public InvertedGenerationalDistance(double p) {
      this.p = 1 / p;
    }



    /// <summary>
    /// 
    /// </summary>
    /// <param name="front"></param>
    /// <param name="optimalFront"></param>
    /// <param name="p"></param>
    /// <returns></returns>
    public static double GetDistance(RealVector[] front, RealVector[] optimalFront, double p) {
      return new InvertedGenerationalDistance(p).Compare(front, optimalFront);
    }

    public double Compare(RealVector[] front, RealVector[] optimalFront) {
      //TODO build a kd-tree, sort the array, do someting intelligent here
      double sum = 0;
      if (front.Length == 0 || optimalFront.Length == 0) throw new Exception("Both Fronts need to contain at least one point");
      foreach (RealVector r in optimalFront) {
        sum += minDistance(r, front);
      }
      return Math.Pow(sum, p) / optimalFront.Length;
    }

    private double minDistance(RealVector point, RealVector[] list) {
      //TODO inefficient
      double min = Double.MaxValue;
      foreach (RealVector r in list) {
        if (r == point) continue;
        double d = 0;
        for (int i = 0; i < r.Length; i++) {
          d += (point[i] - r[i]) * (point[i] - r[i]);
        }
        min = Math.Min(d, min);
      }
      return Math.Sqrt(min);
    }

  }
}
